package com.framsticks.gui;

import org.apache.log4j.Logger;


import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * @author Piotr Sniegowski
 */
@SuppressWarnings("serial")
public abstract class ModifiablePanel extends Panel {

	private static final Logger log = Logger.getLogger(ModifiablePanel.class.getName());

	/**
	 * Pane to which components will be added.
	 */

	protected final JLabel label;
	protected final JButton applyButton;
	protected final JPanel centerPanel;

	protected Component contentComponent;

	public ModifiablePanel(Panel.Parameters parameters) {
		super(parameters);
		log.debug("create panel for type: " + className);



		JPanel pageEndPanel = new JPanel();
		pageEndPanel.setLayout(new BoxLayout(pageEndPanel, BoxLayout.X_AXIS));
		pageEndPanel.add(Box.createHorizontalGlue());

		applyButton = new JButton("Apply");
		applyButton.setName("apply");

		applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applyChanges();
			}
		});
		pageEndPanel.add(applyButton);

		pageEndPanel.add(Box.createHorizontalStrut(10));
		pageEndPanel.setPreferredSize(new Dimension(0, 30));

		label = new JLabel();
		centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(Box.createHorizontalStrut(10), BorderLayout.LINE_START);
		centerPanel.add(Box.createHorizontalStrut(10), BorderLayout.LINE_END);
		centerPanel.add(label, BorderLayout.PAGE_START);


		centerPanel.add(pageEndPanel, BorderLayout.PAGE_END);

		this.setLayout(new BorderLayout());
		this.add(centerPanel, BorderLayout.CENTER);
		//this.add(new ViewerTest(), BorderLayout.PAGE_END);
	}

	protected abstract void applyChanges();

	protected void setupContentComponent(Component contentComponent) {
		this.contentComponent = contentComponent;
		centerPanel.add(contentComponent, BorderLayout.CENTER);
	}

}
